//
// (C) Copyright 2003-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted,
// provided that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

 
using System;
using System.Data;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Autodesk.Revit.ApplicationServices;
using Autodesk.Revit.DB;
using Autodesk.Revit.DB.Events;
using Autodesk.Revit.UI;
using Autodesk.Revit.UI.Events;
using System.IO;

namespace SRD204219
{
    [Autodesk.Revit.Attributes.Regeneration(Autodesk.Revit.Attributes.RegenerationOption.Manual)]
    [Autodesk.Revit.Attributes.Transaction(Autodesk.Revit.Attributes.TransactionMode.Automatic)]
   public class SaveAsEventApplication : IExternalApplication
   {

      #region IExternalApplication Members

        

      public Result OnShutdown(UIControlledApplication application)
      {
         return Result.Succeeded;
      }

      public Result OnStartup(UIControlledApplication application)
      {
         //MessageBox.Show("Build: " + application.ControlledApplication.VersionBuild);

         application.ControlledApplication.DocumentSavingAs += ExDocumentSavingAs;
         application.ControlledApplication.DocumentSavedAs += ExDocumentSavedAs;

         return Result.Succeeded;
      }

      #endregion

      bool reentering = false;

      private void ExDocumentSavingAs(Object sender, DocumentSavingAsEventArgs args)
      {
          try
          {
              if (!reentering)
              {
                  Document document = args.Document;
                  args.Cancel = true;
                  reentering = true;
                  UIApplication uiApp = new UIApplication(document.Application);
                  uiApp.Idling += ExIdling;
              }
          }
          catch (System.Exception ex)
          {
              TaskDialog.Show("Exception in SavingAs", ex.ToString());
          }
  
      }

        private void ExDocumentSavedAs(Object sender, DocumentSavedAsEventArgs args)
        {
            try
            {
                if (args.Status != EventStatus.Cancelled)
                {
                    reentering = false;
                }
            }
            catch (System.Exception ex)
            {
                TaskDialog.Show("Exception in SavedAs", ex.ToString());
            }
        }

        private void ExIdling(Object sender, IdlingEventArgs args)
        {
            Autodesk.Revit.ApplicationServices.Application app = (Autodesk.Revit.ApplicationServices.Application)sender;
            UIApplication uiApp = new UIApplication(app);
            Document document = uiApp.ActiveUIDocument.Document;
            document.SaveAs(@"c:\foo.rvt", true);
            uiApp.Idling -= ExIdling;
        }
 
      private TextWriter m_writer;

   }
}
